import net.tinyos.message.Message;
import net.tinyos.message.MoteIF;
import net.tinyos.packet.BuildSource;
import net.tinyos.packet.PhoenixSource;
import net.tinyos.util.PrintStreamMessenger;

import java.io.IOException;
import java.util.*;

/**
 * Classe encarregada del control dels nodes. Els afegeix a una llista en el moment que demanen 
 * la seva configuració, els inicialitza, guarda els valors de les configuracions, els presenta
 * per pantalla i controla el funcionament dels nodes mitjançant el temps de recepció dels missatges
 * de bateria.
 * @author jterre
 *
 */
	public class ControladorNodes extends ReceptorMissatgesBat {
	
		int mostreigTemp;
		int llindarTemp;
		int mostreigBat;
		int llindarBat;
		long periode;
		Timer timer;
		ArrayList<Node> nodes;
		DemConfigMsg dcm = new DemConfigMsg();
		
	/**
	 * Constructor del controlador de nodes.
	 * @param mTemp
	 * @param lTemp
	 * @param mBat
	 * @param lBat
	 */
	public ControladorNodes(int mTemp, int lTemp, int mBat, int lBat) {
		super();
		this.mostreigTemp = mTemp;
		this.llindarTemp = lTemp;
		this.mostreigBat = mBat;
		this.llindarBat = lBat;
		this.nodes = new ArrayList<Node>();
		this.periode = 1000;
		this.timer = new Timer();
		this.timer.scheduleAtFixedRate(this, 30000, periode); // Deixem 30" abans de començar a controlar els nodes
		
		mif.registerListener(dcm, this);
	}
	
	/**
	 * Lectura dels missatges dels nodes demanant la seva configuració per inicialitzar-los i
	 * de bateria pel control del seu  funcionament.
	 */
	@Override
	public void messageReceived(int to, Message m) { 
		
		if(m instanceof DemConfigMsg){
			DemConfigMsg dcm = (DemConfigMsg) m;
			this.nodeId = dcm.get_nodeid();
			Node n = trobarNode(this.nodeId);
			if(n == null){
				n = new Node(this.nodeId, this.mostreigTemp, this.llindarTemp, 
							 this.mostreigBat, this.llindarBat);
				this.nodes.add(n);
				n.setTempsRecepcio(System.currentTimeMillis());
				System.out.println("\nAfegit node " + n.getNodeId());	
			}
			
			this.inicialitzacioNode(n);
			if(n != null){
				n.setTempsRecepcio(System.currentTimeMillis());	
			}
		}
		
		if(m instanceof BatMsg){
			BatMsg bm = (BatMsg) m;
			this.nodeId = bm.get_nodeid();
			Node n = trobarNode(this.nodeId);
			if(n != null){
				n.setTempsRecepcio(System.currentTimeMillis());	// guarda el temps de recepció del missatge
			}
		}
	}
	
	/**
	 * Envia la configuració inicial al node n.
	 * @param n
	 */
	public void inicialitzacioNode(Node n){
		TransConfigBat tcb = new TransConfigBat(0, n.getMostreigBat(), n.getLlindarBat());
		tcb.run();
		TransConfigTemp tct = new TransConfigTemp(nodeId, n.getMostreigTemp(), n.getLlindarTemp());
		tct.run();
	}

	/**
	 * Recorregut per la llista de nodes comparant el temps de recepció de l'últim missatge amb el 
	 * temps actual. Si la diferència és molt elevada dona un avís.
	 */
	public void run(){
		long tempsActual = System.currentTimeMillis();
		Iterator<Node> it = nodes.iterator();
		Node n = null;
		long diferencia;
		
		while(it.hasNext()){
			n = it.next();
			diferencia =  tempsActual - n.getTempsRecepcio();
			if((int)diferencia > n.getMostreigBat() * 3){
				System.out.print("\nNode " + n.getNodeId() + ": retard transmissió " + diferencia + " ms.");
			}
		}
	}
	
	/**
	 * Guarda a la llista de nodes una nova configuració.
	 * @param nId
	 * @param mT
	 * @param lT
	 * @param mB
	 * @param lB
	 */
	public void novaConfiguracio(int nId, int mT, int lT, int mB, int lB){
		Iterator<Node> it = nodes.iterator();
		Node n = null;
		this.mostreigBat = mB;
		if(nId == 0){		// configuració de bateria: la mateixa per tots els nodes.
			while(it.hasNext()){
				n = it.next();
				n.setMostreigBat(mB);
				n.setLlindarBat(lB);
				n.setTempsRecepcio(System.currentTimeMillis());
			}
			if(!it.hasNext()){
				System.out.println("\n*** Nodes no afegits. Polsi el botó RESET del nodes. ***\n");
			}
		}else{				// configuració de temperatura: una per cada node.
			n = this.trobarNode(nId);
			if(n != null){
				n.setMostreigTemp(mT);
				n.setLlindarTemp(lT);
			}else{
				System.out.println("\n*** Node no afegit. Polsi el botó RESET del node. ***\n");
			}
		}	
	}
	
	/**
	 * Cerca a la llista de nodes.
	 * @param nodeId
	 * @return un node
	 */
	public Node trobarNode(int nodeId){
		Iterator<Node> it = nodes.iterator();
		Node n = null;
		boolean trobat = false;
		
		while(it.hasNext() && !trobat){
			n = it.next();
			if(nodeId == n.getNodeId()){
				trobat = true;
			}else{
				n = null;
			}
		}
		return n;
	}
	
	/**
	 * Presentació per pantalla dels valors de configuració dels nodes.
	 */
	public void printConfiguracions(){
		Iterator<Node> it = nodes.iterator();
		Node n;
		System.out.println("NodeId \t MostreigTemp \t LlindarTemp \t MostreigBat \t LlindarBat");
		System.out.println("===================================================================");
		while(it.hasNext()){
			n = it.next();
			System.out.print("   " + n.getNodeId() + "\t");
			System.out.print("     " + n.getMostreigTemp() + "\t");
			double lecturaVoltsTemp = passarAVolts(2560,n.getLlindarTemp());
			double temperaturaCentigrads = (lecturaVoltsTemp-0.5)/0.01;
			System.out.print("     " + formatejarDouble(temperaturaCentigrads) + "ºC" + "\t");
			System.out.print("     " + n.getMostreigBat()/1.024 + "\t");
			double lecturaVoltsBat = passarAVolts(2560,n.getLlindarBat())*2;
			System.out.print(formatejarDouble(lecturaVoltsBat) + " V" + "\t \n");
		}
		System.out.println();
	}
}
